#!/usr/bin/env python3
from functools import reduce


def check_filename(filepath, extension=".txt"):
    """
    Take a path to a file and ensure the filename has the right extension, else add the extension.
    :param str filepath: The path to the file.
    :param str extension: Extension for the given file. If wrong extension in filepath then raise error.
    Extension must start with '.'.
    :return str: The filepath with the correct extension.
    """
    if extension[0] != ".":
        raise ValueError("Parameter 'extension'  must start with a period e.g. '.txt'.")
    file_extension = filepath[-(len(extension)):].lower()  # get last part of filename to check extension
    extension = extension.lower()
    if file_extension != extension:
        # remove any trailing slashes at end of filepath
        while filepath[-1] == "/" or filepath[-1] == "\\":
            filepath = filepath[0:-1]
        filepath += extension
    return filepath


def convert_char_to_int(char, limit=256):
    """
    Take a character and return an integer value while ensuring the values returned do not go above the limit.
    :param str char: A single character.
    :param int limit: An integer representing the largest value which will start requiring reducing char value.
    (default=256).
    :return int: A number between 1 (0=NULL) and the limit that represents the int value of the char.
    """
    value = ord(char) % limit
    value = 1 if value == 0 else value
    return value


def get_image_size(text_length):
    """
    Take the length of text to be encoded and get the width, height of the resulting image.
    Must account for a null terminator at the end of the image.
    Tries to get the best resolution so that width and height values are as close as possible.
    :param int text_length: The length of text to be encoded as an image.
    :return (int, int): width, height of the image.
    """
    true_length = text_length  # True length must be an even number and include the null terminator(s) at the end.
    if text_length % 2 == 0:  # even length
        true_length = true_length + 2  # add 2 null terminators at the end
    else:  # odd length of text
        true_length += 1

    f = _factors(true_length)
    width, height = 0, 0
    if len(f) % 2 != 0:  # odd number of factors, middle value will give both width, height
        width = height = f[len(f) // 2]
    else:  # even number of factors, get middle 2 values
        while len(f) != 2:
            f.pop(0)  # remove first and last elements
            f.pop()
        width, height = f[1], f[0]  # width should be larger than height
    return width, height


def _factors(number):
    """
    Return a list of sorted numbers that can be multiplied to get the parameter (i.e. factors of parameter number).
    :param int number: Must be a positive number.
    :return [int,int]: List of factors of parameter 'number' sorted in ascending order.
    """
    if number < 0:
        raise ValueError("Parameter 'number' must be a positive value.")
    return sorted(set(reduce(list.__add__,
                             ([i, number // i] for i in range(1, int(number ** 0.5) + 1) if number % i == 0))))


if __name__ == "__main__":
    pass



#!/usr/bin/env python3

"""
text_to_image allows you to encode text data in to a grayscale image and decode data in a garysacle image back to text.

You can encode a string using encode("text", "image_path.png", limit=256)
You can encode a plain text file using encode_file("path_to_text_file.txt", "image_path.png", limit=256)

You can decode an image to a string using decode("path_to_image.png")
You can decode an image to a string using decode_to_file("path_to_image.png", "output_file_path.txt")

Text files must have a '.txt' extension and images must have a '.png' file extension.

The limit parameter refers to the pixel value limit. The default is 256 allowing for values between 0 and 255.
If the value is greater than limit it will be wrapped around i.e. value = number % limit. When the value is equal to
the limit the new value will be 1.
This means that character encoding schemes that use the same amount of values or fewer can encode all their data in the
image without any loss i.e. 256 means 8 bit or less character encoding schemes work without loss e.g. ASCII.

Image sizes are automatically set using the length of text input.
The last pixel (or last 2 when text length is even) in an image will always have a value of 0.

"""

from .encode import encode   # only these 4 functions needed
from .encode import encode_file
from .decode import decode
from .decode import decode_to_file



#!/usr/bin/env python3
from os import path
import sys
import argparse
import logging

from PIL import Image


from text_to_image.utilities import check_filename
from text_to_image.utilities import convert_char_to_int
from text_to_image.utilities import get_image_size


def encode(text, image_path, limit=256):
    """
    Take a string of text and encode it into an 8-bit grayscale image.
    :param str text: Text may be ASCII or UTF-8 but limit value must be changed accordingly.
    :param str image_path: Path to image file. Should have a '.png' extension or no extension
    :param int limit: The value limit for each pixel. 256 = 8 bit meaning all character encoding schemes using 8 or
    fewer bit can be encoded. If limit is 65536 then character encoding schemes using 16 bits or less can be applied
    e.g. UTF-8. When a character is used from a character set greater than the limit, the character value will be
    divided by the limit value. e.g. limit=256 character=Ĭ (value=300), resulting value will be 44. For values equal to
    the limit, the resulting value will be 1 to avoid NULL within the encoded data. Limit is the number of possible
    values in decimal from 1 to a max value. (default=256 i.e. 8 bit pixels/ 1- 256 means 255 possible values)
    :return str:  The path to the image produced.
    """
    if type(text) is not str:
        raise TypeError("Parameter 'text' must be a string.")
    text_length = len(text)
    size = get_image_size(text_length)
    result_path = check_filename(image_path, extension=".png")

    img = Image.new("L", size)  # grayscale, blank black image
    ind = 0
    for row in range(0, size[0]):
        for col in range(0, size[1]):
            if ind < text_length:  # only change pixel value for length of text
                pixel_value = convert_char_to_int(text[ind], limit=limit)
                img.putpixel((row, col), pixel_value)
                ind += 1
            else:  # end of text, leave remaining pixel(s) black to indicate null
                break
    img.save(result_path)
    return result_path


def encode_file(file_path, image_path, limit=256):
    """
    Take a plain text file and encode it to a grayscale image.
    :param str file_path: Path a valid plain text file with text to be encoded. Must have a .txt extension.
    :param str image_path: Path to image to be created. Will be a png image.
    :param int limit: The value limit for each pixel. A value of 256 means 8 bit pixels and any character encoding
    scheme using fewer bits can be used. Characters with a value greater than limit will be divided by limit while
    characters equal to limit will become 1. See encode() for more information. (default = 256 i.e. 8 bit pixels).
    :return str: Path to image produced
    """
    if not path.isfile(file_path):
        raise FileExistsError("The file {0} does not exist. Cannot encode a non-existent text file.".format(file_path))
    if file_path[-4:].lower() != ".txt":
        raise TypeError("File {0} must be a plain text file with a '.txt' file extension".format(file_path))
    with open(file_path, "r") as f:
        text = f.read()

    return encode(text, image_path, limit=limit)


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    parser = argparse.ArgumentParser(description="Encode text (provided or from a text file) to an image.")
    parser.add_argument("image_path", action="store", help="Filename/path for the output image with the encoded text.",
                        type=str)
    text_or_file_group = parser.add_mutually_exclusive_group()
    text_or_file_group.add_argument("-t", "--text", action="store", help="Text to be encoded.", type=str)
    text_or_file_group.add_argument("-f", "--file", action="store", help="Path to plain text file to be encoded.",
                                    type=str)
    parser.add_argument("-l", "--limit", action="store", help="Decimal value limit for pixel value (1 to max)",
                        type=int)
    args = parser.parse_args()
    if args.file is None and args.text is None:
        logging.error("You must either provide text to be encoded or a path to a text file with text to be encoded.")
        sys.exit(1)

    pixel_value_limit = 256
    if args.limit is not None:
        pixel_value_limit = args.limit

    output_file = ""  # path to created image file
    if args.text is not None:
        output_file = encode(args.text, args.image_path, limit=pixel_value_limit)
    if args.file is not None:
        output_file = encode_file(args.file, args.image_path, limit=pixel_value_limit)
    logging.info("Image '{0}' has been created".format(output_file))



#!/usr/bin/env python3

from setuptools import setup
from os import path


here = path.abspath(path.dirname(__file__))


with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="text_to_image",
    version="0.0.2",
    description="Convert your text to a grayscale image and back.",
    long_description=long_description,
    license="MIT",

    url="https://github.com/akapila011/Text-to-Image",
    author="Abhishek Kapila",
    author_email="akapila011@gmail.com",

    packages=["text_to_image"],
    classifiers=["Development Status :: 3 - Alpha",
                 "Intended Audience :: Developers",
                 "Intended Audience :: End Users/Desktop",
                 "Topic :: Scientific/Engineering :: Visualization",
                 "Topic :: Security :: Cryptography",
                 "Topic :: Multimedia :: Graphics",
                 "Programming Language :: Python :: 3",
                 "Natural Language :: English"],
    keywords="Graphics Encoding Decoding Stenography",
    install_requires=["Pillow"],
    python_requires='>=3'
)



#!/usr/bin/env python3
from os import path
import sys
import argparse
import logging

from PIL import Image


from text_to_image.utilities import check_filename


def decode(image_path):
    """
    Decode an image by converting pixel values back to characters.
    :param str image_path: Path to a png image file.
    :return str: Decoded text.
    """
    if not path.isfile(image_path):
        raise FileExistsError("Image file {0} does not exist. Cannot decode a nonexistent image".format(image_path))
    img = Image.open(image_path)
    decoded_text = ""
    for row in range(0, img.size[0]):
        for col in range(0, img.size[1]):
            pixel_value = img.getpixel((row, col))
            if pixel_value != 0:  # ignore 0 (NULL) values
                decoded_text += chr(pixel_value)
    return decoded_text


def decode_to_file(image_path, file_path):
    """
    Take an encoded png image and produce a text file with the decoded text.
    :param str image_path: Path to a png image with encoded text to be extracted.
    :param str file_path: Path to file where decoded text will be stored. Should be a plain text file '.txt'.
    :return str: Path to output text file.
    """
    if not path.isfile(image_path):
        raise FileExistsError("Image file {0} does not exist. Cannot decode a nonexistent image".format(image_path))
    if image_path[-4:].lower() != ".png":
        raise TypeError("Image {0} must be a png image file with a '.png' file extension".format(image_path))
    file_path = check_filename(file_path)

    decoded_text = decode(image_path)
    with open(file_path, "w") as f:
        f.write(decoded_text)
    return file_path


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    parser = argparse.ArgumentParser(description="Decode text from an image and output to console to a text file.")
    parser.add_argument("image_path", action="store", help="Filename/path for the input image with the encoded text.",
                        type=str)
    parser.add_argument("-f", "--file", action="store", help="Path to text file where decoded text should be stored.",
                        type=str)
    args = parser.parse_args()

    if args.file is None:
        print(decode(args.image_path))
    else:
        output_file = decode_to_file(args.image_path, args.file)
        logging.info("File '{0}' has been created with the decoded text".format(output_file))
